@@ -3,19 +3,17 @@
 .. seealso:: https://github.com/geneontology/obographs
 """
 
+import itertools as itt
 import logging
 from collections import defaultdict
 from operator import attrgetter
 from typing import Any, Iterable, List, Mapping, Optional, Set, Tuple, Union
 
-import bioregistry
-from bioregistry import manager
-from curies import Converter
-from pydantic import BaseModel
-from tqdm import tqdm
+from bioregistry import curie_to_str, get_default_converter, manager
+from pydantic import BaseModel, Field
+from tqdm.auto import tqdm
 from typing_extensions import Literal
 
-from . import upgrade
 from .relations import ground_relation
 
 __all__ = [
@@ -36,11 +34,10 @@ OBO_URI_PREFIX = "http://purl.obolibrary.org/obo/"
 OBO_URI_PREFIX_LEN = len(OBO_URI_PREFIX)
 IDENTIFIERS_HTTP_PREFIX = "http://identifiers.org/"
 IDENTIFIERS_HTTPS_PREFIX = "https://identifiers.org/"
+PROVENANCE_PREFIXES = {"pubmed", "pmc", "doi", "arxiv", "biorxiv"}
 
 MaybeCURIE = Union[Tuple[str, str], Tuple[None, None]]
 
-converter = Converter.from_reverse_prefix_map(manager.get_reverse_prefix_map(include_prefixes=True))
-
 
 class StandardizeMixin:
     """A mixin for classes representing standardizable data."""
@@ -73,19 +70,20 @@ class Property(BaseModel, StandardizeMixin):
         """Get the predicate's CURIE or error if unparsable."""
         if self.pred_prefix is None or self.pred_identifier is None:
             raise
-        return bioregistry.curie_to_str(self.pred_prefix, self.pred_identifier)
+        return curie_to_str(self.pred_prefix, self.pred_identifier)
 
     @property
     def val_curie(self) -> str:
         """Get the value's CURIE or error if unparsable."""
         if self.val_prefix is None or self.val_identifier is None:
             raise
-        return bioregistry.curie_to_str(self.val_prefix, self.val_identifier)
+        return curie_to_str(self.val_prefix, self.val_identifier)
 
     def standardize(self):
         """Standardize this property."""
-        self.pred_prefix, self.pred_identifier = _parse_uri(self.pred)
-        self.val_prefix, self.val_identifier = _parse_uri(self.val)
+        self.val = self.val.replace("\n", " ")
+        self.pred_prefix, self.pred_identifier = _parse_uri_or_curie_or_str(self.pred)
+        self.val_prefix, self.val_identifier = _parse_uri_or_curie_or_str(self.val)
         self.standardized = True
 
 
@@ -95,12 +93,22 @@ class Definition(BaseModel):
     val: str
     # Just a list of CURIEs/IRIs
     xrefs: Optional[List[str]]
+    standardized: bool = False
+
+    def standardize(self) -> None:
+        """Standardize the xref."""
+        if self.xrefs:
+            curies = [_clean_uri_or_curie_or_str(xref, keep_invalid=False) for xref in self.xrefs]
+            self.xrefs = [curie for curie in curies if curie]
+        self.standardized = True
 
 
 class Xref(BaseModel, StandardizeMixin):
     """Represents a cross-reference."""
 
     val: str
+    # TODO ask the obo graph people to update the data model and include xref types
+    pred: str = Field(default="oboinowl:hasDbXref")
     prefix: Optional[str]
     identifier: Optional[str]
     standardized: bool = False
@@ -110,15 +118,11 @@ class Xref(BaseModel, StandardizeMixin):
         """Get the xref's CURIE."""
         if self.prefix is None or self.identifier is None:
             raise ValueError(f"can't parse xref: {self.val}")
-        return bioregistry.curie_to_str(self.prefix, self.identifier)
+        return curie_to_str(self.prefix, self.identifier)
 
     def standardize(self) -> None:
         """Standardize the xref."""
-        # TODO now use omni-parser
-        if self.val.startswith("http") or self.val.startswith("https"):
-            self.prefix, self.identifier = _parse_uri(self.val)
-        else:
-            self.prefix, self.identifier = bioregistry.parse_curie(self.val)
+        self.prefix, self.identifier = _parse_uri_or_curie_or_str(self.val)
         self.standardized = True
 
 
@@ -127,16 +131,18 @@ class Synonym(BaseModel, StandardizeMixin):
 
     pred: str
     val: str
-    # Just a list of CURIEs/IRIs
-    xrefs: List[str]
     synonymType: Optional[str]  # noqa:N815
     standardized: bool = False
+    # Just a list of CURIEs/IRIs
+    xrefs: List[str] = Field(default_factory=list)
 
     def standardize(self):
         """Standardize the synoynm."""
-        self.pred = _clean_uri(self.pred, keep_invalid=True)  # type:ignore
+        self.pred = _clean_uri_or_curie_or_str(self.pred, keep_invalid=True)  # type:ignore
         if self.synonymType:
-            self.synonymType = _clean_uri(self.synonymType, keep_invalid=True)  # type:ignore
+            self.synonymType = _clean_uri_or_curie_or_str(
+                self.synonymType, keep_invalid=True
+            )  # type:ignore
         self.standardized = True
 
 
@@ -168,16 +174,16 @@ class Edge(BaseModel):
     def parse_curies(self) -> Tuple[MaybeCURIE, MaybeCURIE, MaybeCURIE]:
         """Get parsed CURIEs for this relationship."""
         return (
-            bioregistry.parse_curie(self.sub),
-            bioregistry.parse_curie(self.pred),
-            bioregistry.parse_curie(self.obj),
+            _parse_uri_or_curie_or_str(self.sub),
+            _parse_uri_or_curie_or_str(self.pred),
+            _parse_uri_or_curie_or_str(self.obj),
         )
 
     def standardize(self):
         """Standardize the edge."""
-        self.sub = _clean_uri(self.sub, keep_invalid=True)
-        self.pred = _clean_uri(self.pred, keep_invalid=True, debug=True)
-        self.obj = _clean_uri(self.obj, keep_invalid=True)
+        self.sub = _clean_uri_or_curie_or_str(self.sub, keep_invalid=True)
+        self.pred = _clean_uri_or_curie_or_str(self.pred, keep_invalid=True, debug=True)
+        self.obj = _clean_uri_or_curie_or_str(self.obj, keep_invalid=True)
 
 
 def _help_get_properties(self, pred: Union[str, List[str]]) -> List[str]:
@@ -187,7 +193,7 @@ def _help_get_properties(self, pred: Union[str, List[str]]) -> List[str]:
         pred = [pred]
     # print(self.meta.basicPropertyValues, pred)
     return [
-        bioregistry.normalize_curie(prop.val_curie) if prop.val_prefix else prop.val
+        manager.normalize_curie(prop.val_curie) if prop.val_prefix else prop.val
         for prop in self.meta.basicPropertyValues or []
         if any(prop.pred == p for p in pred)
     ]
@@ -206,7 +212,7 @@ class Node(BaseModel, StandardizeMixin):
 
     def standardize(self) -> None:
         """Ground the node to a standard prefix and luid based on its id (URI)."""
-        self.prefix, self.luid = _parse_uri(self.id)
+        self.prefix, self.luid = _parse_uri_or_curie_or_str(self.id)
 
         if self.meta:
             for prop in self.meta.basicPropertyValues or []:
@@ -215,6 +221,9 @@ class Node(BaseModel, StandardizeMixin):
             for synonym in self.meta.synonyms or []:
                 synonym.standardize()
 
+            if self.meta.definition:
+                self.meta.definition.standardize()
+
             if self.meta.xrefs:
                 xrefs: List[Xref] = []
                 seen: Set[Tuple[str, str]] = set()
@@ -230,16 +239,15 @@ class Node(BaseModel, StandardizeMixin):
                     seen.add(pair)
                     xrefs.append(xref)
                 self.meta.xrefs = sorted(xrefs, key=attrgetter("prefix"))
-        # tqdm.write("\t".join((self.curie, *(x.curie for x in self.xrefs))))
-        # TODO add xrefs from definition into node if the are "real" CURIEs
+
         self.standardized = True
 
     @property
     def curie(self) -> str:
         """Get the CURIE string representing this node or error if not normalized."""
         if self.prefix is None or self.luid is None:
-            raise ValueError("can not give curie for node")
-        return bioregistry.curie_to_str(self.prefix, self.luid)
+            raise ValueError(f"can not give curie for node {self.id}")
+        return curie_to_str(self.prefix, self.luid)
 
     @property
     def deprecated(self) -> bool:
@@ -262,6 +270,14 @@ class Node(BaseModel, StandardizeMixin):
             return self.meta.xrefs
         return []
 
+    @property
+    def properties(self) -> List[Property]:
+        """Get the properties for this node."""
+        if not self.meta or self.meta.basicPropertyValues is None:
+            return []
+        # TODO filter out ones grabbed by other getters
+        return self.meta.basicPropertyValues
+
     @property
     def replaced_by(self) -> Optional[str]:
         """Get the identifier that this node was replaced by."""
@@ -269,7 +285,7 @@ class Node(BaseModel, StandardizeMixin):
         rv = self._get_property(preds)
         if not rv:
             return None
-        return bioregistry.normalize_curie(rv)
+        return manager.normalize_curie(rv)
 
     @property
     def alternative_ids(self) -> List[str]:
@@ -279,7 +295,14 @@ class Node(BaseModel, StandardizeMixin):
             "oboinowl:hasAlternativeId",
             "oboInOwl:hasAlternativeId",
         ]
-        return [bioregistry.normalize_curie(curie) for curie in self._get_properties(preds)]
+        rv = []
+        for curie in self._get_properties(preds):
+            norm_curie = manager.normalize_curie(curie)
+            if norm_curie:
+                rv.append(norm_curie)
+            else:
+                logger.warning("could not parse CURIE: %s", curie)
+        return rv
 
     @property
     def namespace(self) -> Optional[str]:
@@ -321,7 +344,27 @@ class Node(BaseModel, StandardizeMixin):
 
     def parse_curie(self) -> MaybeCURIE:
         """Parse the identifier into a pair, assuming it's a CURIE."""
-        return bioregistry.parse_curie(self.id)
+        return _parse_uri_or_curie_or_str(self.id)
+
+    @property
+    def definition_provenance(self) -> List[str]:
+        """Get the provenance CURIEs for the definition."""
+        if self.meta and self.meta.definition and self.meta.definition.xrefs:
+            return self.meta.definition.xrefs
+        return []
+
+    def get_provenance(self) -> List[str]:
+        """Get provenance CURIEs from definition and xrefs."""
+        return list(
+            itt.chain(
+                (
+                    curie
+                    for curie in self.definition_provenance
+                    if curie.split(":")[0] in PROVENANCE_PREFIXES
+                ),
+                (xref.curie for xref in self.xrefs if xref.prefix in PROVENANCE_PREFIXES),
+            )
+        )
 
 
 class Graph(BaseModel, StandardizeMixin):
@@ -471,28 +514,38 @@ class Graph(BaseModel, StandardizeMixin):
         xrefs = {}
         for node in self.nodes:
             for xref in node.xrefs:
-                xref_prefix, xref_identifier = bioregistry.parse_curie(xref.val)
+                xref_prefix, xref_identifier = _parse_uri_or_curie_or_str(xref.val)
+                if xref_prefix is None or xref_identifier is None:
+                    continue
                 if xref_prefix != prefix:
                     continue
+                if " " in xref_identifier:
+                    tqdm.write(f"node {node.id} with space in xref {xref.val}")
                 xrefs[xref_identifier] = node.id
         return xrefs
 
+    def get_curie_to_name(self) -> Mapping[str, str]:
+        """Get a mapping from CURIEs to names."""
+        return {node.curie: node.lbl for node in self.nodes if node.lbl and node.prefix}
 
-def _parse_uri(s: str, *, debug: bool = False) -> Union[Tuple[str, str], Tuple[None, None]]:
+
+def _parse_uri_or_curie_or_str(
+    s: str, *, debug: bool = False
+) -> Union[Tuple[str, str], Tuple[None, None]]:
     """Ground the node to a standard prefix and luid based on its id (URI)."""
-    prefix, identifier = _compress_uri(s, debug=debug)
+    prefix, identifier = _compress_uri_or_curie_or_str(s, debug=debug)
     if prefix is None:
         return None, None
-    resource = bioregistry.get_resource(prefix)
+    resource = manager.get_resource(prefix)
     if resource is None:
         return None, None
     return resource.prefix, resource.standardize_identifier(identifier)
 
 
-def _clean_uri(s: str, *, keep_invalid: bool, debug: bool = False) -> Optional[str]:
-    prefix, identifier = _parse_uri(s=s, debug=debug)
+def _clean_uri_or_curie_or_str(s: str, *, keep_invalid: bool, debug: bool = False) -> Optional[str]:
+    prefix, identifier = _parse_uri_or_curie_or_str(s=s, debug=debug)
     if prefix is not None and identifier is not None:
-        return bioregistry.curie_to_str(prefix, identifier)
+        return curie_to_str(prefix, identifier)
     elif keep_invalid:
         return s
     else:
@@ -514,12 +567,18 @@ def _parse_obo_rel(s: str, identifier: str) -> Union[Tuple[str, str], Tuple[None
     return None, s
 
 
-def _compress_uri(s: str, *, debug: bool = False) -> Union[Tuple[str, str], Tuple[None, str]]:
-    cv = upgrade.upgrade(s)
+def _compress_uri_or_curie_or_str(
+    s: str, *, debug: bool = False
+) -> Union[Tuple[str, str], Tuple[None, str]]:
+    from .upgrade import insert, upgrade
+
+    s = s.replace(" ", "")
+
+    cv = upgrade(s)
     if cv:
         return cv
 
-    prefix, identifier = converter.parse_uri(s)
+    prefix, identifier = get_default_converter().parse_uri(s)
     if prefix and identifier:
         if prefix == "obo" and "#" in identifier:
             return _parse_obo_rel(s, identifier)
@@ -535,7 +594,7 @@ def _compress_uri(s: str, *, debug: bool = False) -> Union[Tuple[str, str], Tupl
         if s.startswith(x):
             prefix, identifier = ground_relation(s[len(x) :])
             if prefix and identifier:
-                upgrade.insert(s, prefix, identifier)
+                insert(s, prefix, identifier)
                 return prefix, identifier
             elif s not in WARNED:
                 tqdm.write(f"could not parse legacy RO: {s}")
